@@ -1,4 +1,4 @@
-# Copyright (c) 2015 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 from UM.Math.Vector import Vector
@@ -14,6 +14,7 @@ import threading
 import numpy
 import numpy.linalg
 import scipy.spatial
+import scipy.spatial.qhull
 import hashlib
 from time import time
 numpy.seterr(all="ignore") # Ignore warnings (dev by zero)
@@ -43,6 +44,8 @@ Reuse = object()
 class MeshData:
     def __init__(self, vertices=None, normals=None, indices=None, colors=None, uvs=None, file_name=None,
                  center_position=None, zero_position=None, type = MeshType.faces, attributes=None):
+        self._application = None  # Initialize this later otherwise unit tests break
+
         self._vertices = NumPyUtil.immutableNDArray(vertices)
         self._normals = NumPyUtil.immutableNDArray(normals)
         self._indices = NumPyUtil.immutableNDArray(indices)
@@ -51,7 +54,11 @@ class MeshData:
         self._vertex_count = len(self._vertices) if self._vertices is not None else 0
         self._face_count = len(self._indices) if self._indices is not None else 0
         self._type = type
-        self._file_name = file_name  # type: str
+        self._file_name = file_name  # type: Optional[str]
+        if file_name:
+            from UM.Application import Application
+            self._application = Application.getInstance()
+            self._application.getController().getScene().addWatchedFile(file_name)
         # original center position
         self._center_position = center_position
         # original zero position, is changed after transformation
@@ -74,6 +81,14 @@ class MeshData:
                         new_value[attribute_key] = attribute_value
                 self._attributes[key] = new_value
 
+    ##  Triggered when this file is deleted.
+    #
+    #   The file will then no longer be watched for changes.
+    def __del__(self):
+        if self._file_name:
+            if self._application:
+                self._application.getController().getScene().removeWatchedFile(self._file_name)
+
     ## Create a new MeshData with specified changes
     #   \return \type{MeshData}
     def set(self, vertices=Reuse, normals=Reuse, indices=Reuse, colors=Reuse, uvs=Reuse, file_name=Reuse,
@@ -374,12 +389,12 @@ def approximateConvexHull(vertex_data: numpy.ndarray, target_count: int) -> Opti
         return None
 
     # Take the convex hull and keep on rounding it off until the number of vertices is below the target_count.
-    hull_result = scipy.spatial.ConvexHull(vertex_data)
+    hull_result = createConvexHull(vertex_data)
     vertex_data = numpy.take(hull_result.points, hull_result.vertices, axis=0)
 
     while len(vertex_data) > target_count and unit_size <= max_unit_size:
         vertex_data = uniqueVertices(roundVertexArray(vertex_data, unit_size))
-        hull_result = scipy.spatial.ConvexHull(vertex_data)
+        hull_result = createConvexHull(vertex_data)
         vertex_data = numpy.take(hull_result.points, hull_result.vertices, axis=0)
         unit_size *= 2
 
@@ -389,6 +404,17 @@ def approximateConvexHull(vertex_data: numpy.ndarray, target_count: int) -> Opti
     return hull_result
 
 
+def createConvexHull(vertex_data: numpy.ndarray) -> Optional[scipy.spatial.ConvexHull]:
+    hull_result = None
+    try:
+        hull_result = scipy.spatial.ConvexHull(vertex_data)
+    except scipy.spatial.qhull.QhullError:
+        # Can get an error when the model is lower dimensional, use "QJ" is make it full dimensional
+        Logger.log("w", "Loaded model is low-dimensional, apply QJ to make it full dimensional")
+        hull_result = scipy.spatial.ConvexHull(vertex_data, qhull_options="QJ")
+    return hull_result
+
+
 ##  Calculate the normals of this mesh, assuming it was created by using addFace (eg; the verts are connected)
 #
 #   \param vertices \type{narray} list of vertices as a 1D list of float triples
