@@ -1,11 +1,12 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
+import datetime
 import os
 import os.path
 import re
 import shutil
-from typing import List
+from typing import Dict, Generator, List, Optional
 
 from UM.Logger import Logger
 from UM.Platform import Platform
@@ -42,6 +43,8 @@ class Resources:
     ContainerStacks = 10
     ## Location of cached data
     Cache = 11
+    ## Location of plugins
+    Plugins = 12
 
     ## Any custom resource types should be greater than this to prevent collisions with standard types.
     UserType = 128
@@ -49,6 +52,8 @@ class Resources:
     ApplicationIdentifier = "UM"
     ApplicationVersion = "unknown"
 
+    __bundled_resources_path = None #type: Optional[str]
+
     ##  Get the path to a certain resource file
     #
     #   \param resource_type \type{int} The type of resource to retrieve a path for.
@@ -81,7 +86,7 @@ class Resources:
     #   \return A list of absolute paths to resources of the specified type.
     @classmethod
     def getAllResourcesOfType(cls, resource_type: int) -> List[str]:
-        files = {}
+        files = {} #type: Dict[str, List[str]]
         search_dirs = cls.getAllPathsForType(resource_type)
 
         for directory in search_dirs:
@@ -141,8 +146,8 @@ class Resources:
 
     ##  Return a path where a certain resource type can be stored.
     #
-    #   \param type \type{int} The type of resource to store.
-    #   \return \type{string} An absolute path where the given resource type can be stored.
+    #   \param type The type of resource to store.
+    #   \return An absolute path where the given resource type can be stored.
     #
     #   \exception UnsupportedStorageTypeError Raised when writing type is not supported.
     @classmethod
@@ -174,13 +179,13 @@ class Resources:
     #
     #   \param path The path to add.
     @classmethod
-    def addSearchPath(cls, path: str):
+    def addSearchPath(cls, path: str) -> None:
         if os.path.isdir(path) and path not in cls.__paths:
             cls.__paths.append(path)
 
     ##  Remove a resource search path.
     @classmethod
-    def removeSearchPath(cls, path: str):
+    def removeSearchPath(cls, path: str) -> None:
         if path in cls.__paths:
             del cls.__paths[cls.__paths.index(path)]
 
@@ -189,7 +194,7 @@ class Resources:
     #   \param type \type{int} An integer that can be used to identify the type. Should be greater than UserType.
     #   \param path \type{string} The path relative to the search paths where resources of this type can be found./
     @classmethod
-    def addType(cls, resource_type: int, path: str):
+    def addType(cls, resource_type: int, path: str) -> None:
         if resource_type in cls.__types:
             raise ResourceTypeError("Type {0} already exists".format(resource_type))
 
@@ -203,7 +208,7 @@ class Resources:
     #   \param type The type to add a storage path for.
     #   \param path The path to add as storage path. Should be relative to the resources storage path.
     @classmethod
-    def addStorageType(cls, resource_type: int, path: str):
+    def addStorageType(cls, resource_type: int, path: str) -> None:
         if resource_type in cls.__types:
             raise ResourceTypeError("Type {0} already exists".format(resource_type))
 
@@ -242,12 +247,12 @@ class Resources:
     #
     #   \return A sequence of paths where resources might be.
     @classmethod
-    def getSearchPaths(cls):
+    def getSearchPaths(cls) -> Generator[str, None, None]:
         yield from cls.__paths
 
     ##  Remove a custom resource type.
     @classmethod
-    def removeType(cls, resource_type: int):
+    def removeType(cls, resource_type: int) -> None:
         if resource_type not in cls.__types:
             return
 
@@ -259,11 +264,62 @@ class Resources:
         if resource_type in cls.__types_storage:
             del cls.__types_storage[resource_type]
 
+    ##  Performs a factory reset, compressing the current state of configuration
+    #   into an archive and emptying the resource folders.
+    #
+    #   When calling this function, be sure to quit the application immediately
+    #   afterwards, lest the save function write the configuration anew.
+    @classmethod
+    def factoryReset(cls) -> None:
+        config_path = cls.getConfigStoragePath()
+        data_path = cls.getDataStoragePath()
+        cache_path = cls.getCacheStoragePath()
+
+        folders_to_backup = set()
+        folders_to_remove = set()  # only cache folder needs to be removed
+
+        folders_to_backup.add(config_path)
+        folders_to_backup.add(data_path)
+
+        # Only remove the cache folder if it's not the same as data or config
+        if cache_path not in folders_to_backup:
+            folders_to_remove.add(cache_path)
+
+        for folder in folders_to_remove:
+            shutil.rmtree(folder, ignore_errors = True)
+        for folder in folders_to_backup:
+            base_name = os.path.basename(folder)
+            root_dir = os.path.dirname(folder)
+
+            date_now = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
+            idx = 0
+            file_name = base_name + "_" + date_now
+            zip_file_path = os.path.join(root_dir, file_name + ".zip")
+            while os.path.exists(zip_file_path):
+                idx += 1
+                file_name = base_name + "_" + date_now + "_" + str(idx)
+                zip_file_path = os.path.join(root_dir, file_name + ".zip")
+            try:
+                # only create the zip backup when the folder exists
+                if os.path.exists(folder):
+                    # remove the .zip extension because make_archive() adds it
+                    zip_file_path = zip_file_path[:-4]
+                    shutil.make_archive(zip_file_path, "zip", root_dir = root_dir, base_dir = base_name)
+
+                    # remove the folder only when the backup is successful
+                    shutil.rmtree(folder, ignore_errors = True)
+
+                # create an empty folder so Resources will not try to copy the old ones
+                os.makedirs(folder, 0o0755, exist_ok=True)
+
+            except:
+                Logger.logException("e", "Failed to backup [%s] to file [%s]", folder, zip_file_path)
+
     ## private:
 
     # Returns a list of paths where args was found.
     @classmethod
-    def __find(cls, resource_type: int, *args) -> List[str]:
+    def __find(cls, resource_type: int, *args: str) -> List[str]:
         suffix = cls.__types.get(resource_type, None)
         if suffix is None:
             return []
@@ -276,7 +332,7 @@ class Resources:
         return files
 
     @classmethod
-    def _getConfigStorageRootPath(cls):
+    def _getConfigStorageRootPath(cls) -> str:
         # Returns the path where we store different versions of app configurations
         config_path = None
         if Platform.isWindows():
@@ -294,7 +350,7 @@ class Resources:
         return config_path
 
     @classmethod
-    def _getPossibleConfigStorageRootPathList(cls):
+    def _getPossibleConfigStorageRootPathList(cls) -> List[str]:
         # Returns all possible root paths for storing app configurations (in old and new versions)
         config_root_list = [Resources._getConfigStorageRootPath()]
         if Platform.isWindows():
@@ -320,7 +376,7 @@ class Resources:
         return data_root_list
 
     @classmethod
-    def _getDataStorageRootPath(cls):
+    def _getDataStorageRootPath(cls) -> str:
         # Returns the path where we store different versions of app data
         data_path = None
         if Platform.isLinux():
@@ -331,7 +387,7 @@ class Resources:
         return data_path
 
     @classmethod
-    def _getCacheStorageRootPath(cls):
+    def _getCacheStorageRootPath(cls) -> str:
         # Returns the path where we store different versions of app configurations
         cache_path = None
         if Platform.isWindows():
@@ -347,7 +403,7 @@ class Resources:
         return cache_path
 
     @classmethod
-    def __initializeStoragePaths(cls):
+    def __initializeStoragePaths(cls) -> None:
         Logger.log("d", "Initializing storage paths")
         # use nested structure: <app-name>/<version>/...
         if cls.ApplicationVersion == "master" or cls.ApplicationVersion == "unknown":
@@ -388,7 +444,7 @@ class Resources:
     ##  Copies the directories of the latest version on this machine if present, so the upgrade will use the copies
     #   as the base for upgrade. See CURA-3529 for more details.
     @classmethod
-    def _copyLatestDirsIfPresent(cls):
+    def _copyLatestDirsIfPresent(cls) -> None:
         # Paths for the version we are running right now
         this_version_config_path = Resources.getConfigStoragePath()
         this_version_data_path = Resources.getDataStoragePath()
@@ -432,7 +488,7 @@ class Resources:
             shutil.rmtree(suspected_cache_path)
 
     @classmethod
-    def _findLatestDirInPaths(cls, search_path_list, dir_type="config"):
+    def _findLatestDirInPaths(cls, search_path_list: List[str], dir_type: str = "config") -> str:
         # version dir name must match: <digit(s)>.<digit(s)><whatever>
         version_regex = re.compile(r'^[0-9]+\.[0-9]+.*$')
         check_dir_type_func_dict = {
@@ -446,10 +502,7 @@ class Resources:
             if not os.path.exists(search_path):
                 continue
 
-            if check_dir_type_func(search_path):
-                latest_config_path = search_path
-                break
-
+            # Give priority to a folder with files with version number in it
             storage_dir_name_list = next(os.walk(search_path))[1]
             if storage_dir_name_list:
                 storage_dir_name_list = sorted(storage_dir_name_list, reverse=True)
@@ -470,27 +523,33 @@ class Resources:
                     break
             if latest_config_path is not None:
                 break
+
+            # If not, check if there is a non versioned data dir
+            if check_dir_type_func(search_path):
+                latest_config_path = search_path
+                break
+
         return latest_config_path
 
     @classmethod
-    def _isNonVersionedDataDir(cls, check_path):
+    def _isNonVersionedDataDir(cls, check_path: str) -> bool:
         # checks if the given path is (probably) a valid app directory for a version earlier than 2.6
         if not cls.__expected_dir_names_in_data:
             return True
 
         dirs, files = next(os.walk(check_path))[1:]
         valid_dir_names = [dn for dn in dirs if dn in Resources.__expected_dir_names_in_data]
-        return valid_dir_names
+        return len(valid_dir_names) > 0
 
     @classmethod
-    def _isNonVersionedConfigDir(cls, check_path):
+    def _isNonVersionedConfigDir(cls, check_path: str) -> bool:
         dirs, files = next(os.walk(check_path))[1:]
         valid_file_names = [fn for fn in files if fn.endswith(".cfg")]
 
-        return bool(valid_file_names)
+        return len(valid_file_names) > 0
 
     @classmethod
-    def addExpectedDirNameInData(cls, dir_name):
+    def addExpectedDirNameInData(cls, dir_name: str) -> None:
         cls.__expected_dir_names_in_data.append(dir_name)
 
     __expected_dir_names_in_data = []  # type: List[str]
@@ -512,7 +571,8 @@ class Resources:
         DefinitionContainers: "definitions",
         InstanceContainers: "instances",
         ContainerStacks: "stacks",
-    }
+        Plugins: "plugins",
+    } #type: Dict[int, str]
     __types_storage = {
         Resources: "",
         Preferences: "",
@@ -521,4 +581,5 @@ class Resources:
         InstanceContainers: "instances",
         ContainerStacks: "stacks",
         Themes: "themes",
-    }
+        Plugins: "plugins",
+    } #type: Dict[int, str]
