@@ -2,7 +2,7 @@
 # Copyright (c) Thiago Marcos P. Santos
 # Copyright (c) Christopher S. Case
 # Copyright (c) David H. Bronke
-# Uranium is released under the terms of the AGPLv3 or higher.
+# Uranium is released under the terms of the LGPLv3 or higher.
 
 import enum #For the compress parameter of postponeSignals.
 import inspect
@@ -292,7 +292,7 @@ class Signal:
     #   set by the Application instance.
     _app = None  # type: Application
 
-    _signalQueue = None  # type: SignalQueue
+    _signalQueue = None  # type: Application
 
     # Private implementation of the actual emit.
     # This is done to make it possible to freely push function events without needing to maintain state.
